/*****************************************************************************/
/* SmpVSTHost.h: Interface for classes CSmpVSTHost / CSmpEffect              */
/*****************************************************************************/

#if !defined(AFX_SMPVSTHOST_H__EA642421_A9AF_11D5_B6BD_004F4E003207__INCLUDED_)
#define AFX_SMPVSTHOST_H__EA642421_A9AF_11D5_B6BD_004F4E003207__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "CVSTHost.h"

/*****************************************************************************/
/* CSmpEffect class declaration                                              */
/*****************************************************************************/

class CEffectWnd;
class CSmpEffect : public CEffect  
{
public:
	CSmpEffect(CVSTHost *pHost);
	virtual ~CSmpEffect();

public:
	void LeaveCritical();
	void EnterCritical();
	virtual bool Load(const char *name);
    virtual bool Unload();
	virtual long EffDispatch(long opCode, long index=0, long value=0, void *ptr=0, float opt=0.);
	virtual void OnSizeEditorWindow(long width, long height);
    void SetEditWnd(CEffectWnd *pEWnd = 0) { pEditWnd = pEWnd; }
    void SetParmWnd(CEffectWnd *pPWnd = 0) { pParmWnd = pPWnd; }
    unsigned short GetChnMask() { return wChnMask; }
    void SetChnMask(unsigned short mask) { wChnMask = mask; }
    bool OnSetParameterAutomated(long index, float value);
    virtual bool OnUpdateDisplay();
    void SetChunkFile(CString sFile) { sChunkFile = sFile; }
    CString GetChunkFile() { return sChunkFile; }

	void SetInputBuffer(int nBuf, float *pData = 0)
      {
      if (nBuf < pEffect->numInputs)
        // inBufs allocation isn't checked here; this is intentional.
        // We rely on a sensible implementation that doesn't set
        // input buffers before initialization is finished (which
        // would throw out an effect if buffer allocation fails).
        inBufs[nBuf] = pData;
      }

	float * GetOutputBuffer(int nBuf)
      {
      if (nBuf < pEffect->numOutputs)
        // outBufs allocation isn't checked here; this is intentional.
        // We rely on a sensible implementation that doesn't set
        // input buffers before initialization is finished (which
        // would throw out an effect if buffer allocation fails).
        return outBufs[nBuf];
      else
        return 0;
      }

    virtual void EffProcess(long sampleFrames)
      {
      // operate using internally allocated buffers
      CEffect::EffProcess(inBufs, outBufs, sampleFrames);
      TRACE0("effProcess\n");
      }
    virtual void EffProcessReplacing(long sampleFrames)
      {
      // operate using internally allocated buffers
      CEffect::EffProcessReplacing(inBufs, outBufs, sampleFrames);
      TRACE0("effProcessReplacing\n");
      }

protected:
	unsigned short wChnMask;
	int nAllocatedOutbufs;
	int nAllocatedInbufs;
	static int EvalException(int n_except);
    CRITICAL_SECTION cs;

	float ** outBufs;
	float ** inBufs;
	CEffectWnd * pEditWnd;
	CEffectWnd * pParmWnd;
    CString sChunkFile;
};

/*****************************************************************************/
/* CSmpVSTHost class declaration                                             */
/*****************************************************************************/

class CMidiMsg;
class CSmpVSTHost : public CVSTHost  
{
public:
	CSmpVSTHost();
	virtual ~CSmpVSTHost();

public:
	BYTE * CreateMIDISubset(void *pEvData, unsigned short wChnMask = 0xffff);
    void OnSamples(float **pBuffer, int nLength, int nChannels = 2, DWORD dwStamp = 0);
	virtual bool OnProcessEvents(int nEffect, VstEvents* events);
	virtual void OnMidiIn(CMidiMsg &msg);
                                        /* create customized effects         */
    virtual CEffect * CreateEffect() { return new CSmpEffect(this); }
    // overridden callback functions
	virtual long OnAudioMasterCallback(int nEffect, long opcode, long index, long value, void *ptr, float opt);
	virtual bool OnSetParameterAutomated(int nEffect, long index, float value);
	virtual bool OnCanDo(const char *ptr);
    virtual bool OnOpenFileSelector (int nEffect, VstFileSelect *ptr);
    virtual bool OnCloseFileSelector (int nEffect, VstFileSelect *ptr);
    virtual bool OnGetChunkFile(int nEffect, void * nativePath);

protected:
	CRITICAL_SECTION cs;
	int nMidis;
	DWORD * pMidis;
	float * pOutputs[2];
    int nSysEx;
    CMidiMsg *pSysEx;

protected:
	BYTE * CreateMIDIEvents(int nLength, DWORD dwStamp, int nm, int ns);

};

#endif // !defined(AFX_SMPVSTHOST_H__EA642421_A9AF_11D5_B6BD_004F4E003207__INCLUDED_)
