/*****************************************************************************/
/* vsthost.cpp : Defines the class behaviors for the application.            */
/*****************************************************************************/

#include "stdafx.h"
#include "vsthost.h"

#include "MainFrm.h"
#include "ChildFrm.h"

#include "specmidi.h"
#include "mididev.h"
#include "specwave.h"
#include "wavedev.h"
#include "WorkThread.h"

#ifdef __ASIO_H
#include "SpecAsioHost.h"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/*****************************************************************************/
/* CAboutDlg dialog used for App About                                       */
/*****************************************************************************/

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/*===========================================================================*/
/* CVsthostApp class members                                                 */
/*===========================================================================*/

/*****************************************************************************/
/* CVsthostApp message map                                                   */
/*****************************************************************************/

BEGIN_MESSAGE_MAP(CVsthostApp, CWinApp)
	//{{AFX_MSG_MAP(CVsthostApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_FILE_NEW, OnFileNew)
	ON_COMMAND(IDM_MIDIDEV, OnMididev)
	ON_COMMAND(IDM_WAVEDEV, OnWavedev)
	ON_COMMAND(IDM_ENG_START, OnEngStart)
	ON_UPDATE_COMMAND_UI(IDM_ENG_START, OnUpdateEngStart)
	ON_COMMAND(IDM_ENGINE_RESTART, OnEngineRestart)
	ON_UPDATE_COMMAND_UI(IDM_ENGINE_RESTART, OnUpdateEngineRestart)
	ON_COMMAND(IDM_ASIO_CPL, OnAsioCpl)
	ON_UPDATE_COMMAND_UI(IDM_ASIO_CPL, OnUpdateAsioCpl)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/*****************************************************************************/
/* CVsthostApp : constructor                                                 */
/*****************************************************************************/

CVsthostApp::CVsthostApp()
{
bEngRunning = FALSE;
}

/*****************************************************************************/
/* The one and only CVsthostApp object                                       */
/*****************************************************************************/

CVsthostApp theApp;

/*****************************************************************************/
/* InitInstance : application initialization                                 */
/*****************************************************************************/

BOOL CVsthostApp::InitInstance()
{
// Allow XP Look & Feel
InitCommonControls();
CWinApp::InitInstance();

#if 0
SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);
#endif

// Standard initialization
// If you are not using these features and wish to reduce the size
//  of your final executable, you should remove from the following
//  the specific initialization routines you do not need.

#ifdef _AFXDLL
Enable3dControls();			// Call this when using MFC in a shared DLL
#else
Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

// Change the registry key under which our settings are stored.
SetRegistryKey(_T("Seib"));

CCommandLineInfo cmdInfo;
ParseCommandLine(cmdInfo);

#ifdef __ASIO_H
pAsioHost = new CSpecAsioHost;
#else
pAsioHost = 0;
#endif

// To create the main window, this code creates a new frame window
// object and then sets it as the application's main window object.

CMDIFrameWnd* pFrame = new CMainFrame;
m_pMainWnd = pFrame;

// create main MDI frame window
if (!pFrame->LoadFrame(IDR_MAINFRAME))
  return FALSE;

// try to load shared MDI menus and accelerator table
//TODO: add additional member variables and load calls for
//	additional menu types your application may need. 

HINSTANCE hInst = AfxGetResourceHandle();
m_hMDIMenu  = ::LoadMenu(hInst, MAKEINTRESOURCE(IDR_VSTHOSTYPE));
m_hMDIAccel = ::LoadAccelerators(hInst, MAKEINTRESOURCE(IDR_VSTHOSTYPE));

// The main window has been initialized, so show and update it.
pFrame->ShowWindow(m_nCmdShow);
pFrame->UpdateWindow();

wTID = 0;                               /* reset HR timer ID                 */
timeGetDevCaps(&tc, sizeof(tc));        /* get timer capabilities            */
for (int i = 0; i < 2; i++)             /* prepare empty buffers             */
  {
  emptyBuf[i] = new float[44100];
  if (emptyBuf[i])
    for (int j = 0; j < 44100; j++)
      emptyBuf[i][j] = 0.f;
  }
                                        /* create work thread                */
pWorkThread = new CWorkThread(&vstHost);

int nBufSz = GetProfileInt("Settings", "WaveOutBufSize", 4410);
                                        /* open outputs device               */
MidiOut.Open(GetProfileString("Settings", "MidiOut"));
LoadWaveOutDevice(GetProfileString("Settings", "WaveOut"), nBufSz);

#ifdef __ASIO_H
if (pAsioHost)                          /* if ASIO host there, pass thread   */
  pAsioHost->SetWorkThread(pWorkThread);
#endif
                                        /* open input devices                */
MidiIn.Open(GetProfileString("Settings", "MidiIn"),
            OPENDEFAULT,
            OPENDEFAULT,
            CALLBACK_FUNCTION);
MidiIn.SetMidiThru(GetProfileInt("Settings", "MidiThru", FALSE));

if (GetProfileString("Settings", "WaveOut").Left(6).CompareNoCase("ASIO: "))
  LoadWaveInDevice(GetProfileString("Settings", "WaveIn"), nBufSz);

                                        /* if called with effect name, open  */
if (cmdInfo.m_nShellCommand == CCommandLineInfo::FileOpen)
  LoadEffect(cmdInfo.m_strFileName);

if (GetProfileInt("Settings", "Engine", 0))
  EngineStart();

return TRUE;
}

/*****************************************************************************/
/* ExitInstance : called when the application terminates                     */
/*****************************************************************************/

int CVsthostApp::ExitInstance() 
{
FullStop();                             /* stop engine and close devices     */

vstHost.RemoveAll();                    /* remove all loaded effects         */

for (int i = 0; i < 2; i++)             /* delete empty buffers              */
  if (emptyBuf[i])
    delete[] emptyBuf[i];

#ifdef __ASIO_H
if (pAsioHost)
  delete pAsioHost;
#endif

if (pWorkThread)
  delete pWorkThread;

if (m_hMDIMenu != NULL)
  FreeResource(m_hMDIMenu);
if (m_hMDIAccel != NULL)
  FreeResource(m_hMDIAccel);

return CWinApp::ExitInstance();
}

/*****************************************************************************/
/* OnFileNew : called when File / New is selected                            */
/*****************************************************************************/

void CVsthostApp::OnFileNew() 
{
CMainFrame* pFrame = STATIC_DOWNCAST(CMainFrame, m_pMainWnd);

CString sStartAt;
sStartAt = GetProfileString("Load", "Path");
if (sStartAt.GetLength())
  sStartAt += "\\*.*";

CFileDialog dlg(TRUE, NULL, sStartAt);

if (dlg.DoModal() == IDOK)
  {
  sStartAt = dlg.GetPathName();
  sStartAt = sStartAt.Left(sStartAt.ReverseFind('\\'));
  WriteProfileString("Load", "Path", sStartAt);
  LoadEffect(dlg.GetPathName());
  }
}

/*****************************************************************************/
/* OnAppAbout : called to display the About... dialog                        */
/*****************************************************************************/

void CVsthostApp::OnAppAbout()
{
CAboutDlg aboutDlg;
aboutDlg.DoModal();
}

/*****************************************************************************/
/* FillPopup : fills a popup menu with an effect's programs                  */
/*****************************************************************************/

void CVsthostApp::FillPopup(CMenu* pPopupMenu, int nEffect)
{
int i, j;

CEffect *pEffect = vstHost.GetAt(nEffect);
                                        /* remove all old items              */
for (i = pPopupMenu->GetMenuItemCount() - 1; i >= 2; i--)
  {
  if ((pPopupMenu->GetSubMenu(i)) ||
      (pPopupMenu->GetMenuItemID(i) ==
           IDM_EFF_PROGRAM_0))
    pPopupMenu->RemoveMenu(i, MF_BYPOSITION);
  }

if ((pEffect) &&
    (pEffect->pEffect->numPrograms > 1))
  {
  long lProg = pEffect->EffGetProgram();
  bool bProgSet = false;

  int nClass = (pEffect->pEffect->flags & effFlagsIsSynth) ?
      kPlugCategSynth :
      kPlugCategUnknown;
  for (i = 0; i < pEffect->pEffect->numPrograms; i += 16)
    {
    CMenu popup;
    popup.CreatePopupMenu();
    for (j = i; (j < i + 16) && (j < pEffect->pEffect->numPrograms); j++)
      {
      CString sProg;
      char szPgName[256] = "";
      if (!pEffect->EffGetProgramNameIndexed(nClass, j, szPgName))
        {
        bProgSet = true;
        pEffect->EffSetProgram(j);
        pEffect->EffGetProgramName(szPgName);
        if (!*szPgName)
         sprintf(szPgName, "Program %d", j);
        }
      sProg.Format("%d. %s", j, szPgName);
      popup.AppendMenu(MF_STRING, IDM_EFF_PROGRAM_0 + j, sProg);
      }
    CString sSub;
    sSub.Format("Programs %d-%d", i, j-1);
    pPopupMenu->AppendMenu(MF_POPUP | MF_STRING,
                           (UINT)popup.Detach(),
                           sSub);
    }
  if (bProgSet)
    pEffect->EffSetProgram(lProg);
  pPopupMenu->CheckMenuItem(IDM_EFF_PROGRAM_0 + lProg,
                            MF_CHECKED | MF_BYCOMMAND);
  }
}

/*****************************************************************************/
/* LoadEffect : loads a new effect                                           */
/*****************************************************************************/

BOOL CVsthostApp::LoadEffect(LPCSTR lpszFileName)
{
BOOL bWasRunning = bEngRunning;

EngineStop();                           /* stop engine                       */

int nEffect = vstHost.LoadPlugin(lpszFileName);

if (bWasRunning)
  EngineStart();                        /* restart input engine              */

if (nEffect >= 0)
  {
  // create a new MDI child window
  CChildFrame *pChild = (CChildFrame *)CreateChild(RUNTIME_CLASS(CChildFrame),
                                                   IDR_VSTHOSTYPE);
  if (pChild)
    {
    pChild->SetEffect(nEffect);
    return TRUE;
    }
  }

return FALSE;
}

/*****************************************************************************/
/* CreateChild : creats a new MDI child window                               */
/*****************************************************************************/

CMDIChildWnd * CVsthostApp::CreateChild
    (
    CRuntimeClass *pClass,
    UINT nResource,
    HMENU hMenu,
    HACCEL hAccel
    )
{
if (!hMenu)
  hMenu = m_hMDIMenu;
if (!hAccel)
  hAccel = m_hMDIAccel;
CMDIFrameWnd *pMain = (CMDIFrameWnd*)m_pMainWnd;
CMDIChildWnd *pChild = pMain->CreateNewChild(pClass,
      nResource,
      hMenu,
      hAccel);
if (pChild)
  pMain->MDIActivate(pChild);
return pChild;
}

/*****************************************************************************/
/* OnMididev : called to determine the used MIDI devices                     */
/*****************************************************************************/

void CVsthostApp::OnMididev() 
{
CMidiDeviceDialog dlg(AfxGetMainWnd());

dlg.InName = GetProfileString("Settings", "MidiIn");
dlg.OutName = GetProfileString("Settings", "MidiOut");
dlg.bPassThru = GetProfileInt("Settings", "MidiThru", FALSE);
if (dlg.DoModal() == IDOK)
  {
  BOOL bWasRunning = bEngRunning;       /* remember old setting              */
  EngineStop();                         /* stop input engine                 */

  if (!MidiIn.Open(dlg.InName,          /* open input device                 */
                  OPENDEFAULT,
                  OPENDEFAULT,
                  CALLBACK_FUNCTION))
    dlg.InName = "";
  if (!MidiOut.Open(dlg.OutName))       /* open output device                */
    dlg.OutName = "";
  MidiIn.SetMidiThru(dlg.bPassThru);    /* set midi thru                     */
  WriteProfileString("Settings", "MidiIn", dlg.InName);
  WriteProfileString("Settings", "MidiOut", dlg.OutName);
  WriteProfileInt("Settings", "MidiThru", dlg.bPassThru);

  if (bWasRunning)                      /* if it was running before,         */
    EngineStart();                      /* start input engine                */
  }
}

/*****************************************************************************/
/* TimerCallback : high-resolution timer entry point                         */
/*****************************************************************************/

void CALLBACK CVsthostApp::TimerCallback
    (
    UINT uID,
    UINT uMsg,
    DWORD dwUser,
    DWORD dw1,
    DWORD dw2
    )
{
if (dwUser)                             /* pass it on to real app. object    */
  ((CVsthostApp *)dwUser)->HRTimer();
}

/*****************************************************************************/
/* SetTimer : starts / stops a high-resolution timer if no Wave In device    */
/*****************************************************************************/

BOOL CVsthostApp::SetTimer(UINT nMSecs)
{
if ((!nMSecs) && (wTID))                /* if killing a running timer        */
  {
  timeKillEvent(wTID);
  wTID = 0;
  timeEndPeriod(tc.wPeriodMin);
  nTimerMSecs = 0;
  return TRUE;
  }
else                                    /* if starting a HR timer            */
  {
  if (nMSecs < tc.wPeriodMin)           /* look whether possible             */
    return FALSE;

  nTimerMSecs = nMSecs;                 /* remember requested period         */
  nMSecs /= 10;                         /* set HR timer to 1/10 of that      */
  if (nMSecs < tc.wPeriodMin)           /* prohibit going TOO low            */
    nMSecs = tc.wPeriodMin;

  timeBeginPeriod(nMSecs);              /* start using timer services        */
  dwLastTime = timeGetTime();           /* setup for correct start           */
  dwRest = 0;
  wTID = timeSetEvent(                  /* start the timer procedure         */
      nMSecs,          
      (nMSecs > 1) ? nMSecs / 2 : 1,
      TimerCallback,
      (DWORD)this,
      TIME_PERIODIC);
  return !!wTID;                        /* return whether started            */
  }
}

/*****************************************************************************/
/* HRTimer : called when the high-res timer ticks                            */
/*****************************************************************************/

void CVsthostApp::HRTimer()
{
static DWORD dwCurTime;                 /* current time                      */
static DWORD dwOffset;                  /* time offset                       */
static WORD  wClocks;                   /* # clock ticks to send             */
static WORD  i;                         /* clock tick counter                */

dwCurTime = timeGetTime();              /* get current time                  */
dwOffset = dwCurTime -                  /* calculate #msecs since last msg   */
           dwLastTime;
if (dwOffset > dwCurTime)               /* if overflow                       */
  {
  dwLastTime = 0;                       /* reset start tick                  */
  dwOffset = dwCurTime;                 /* this may cause one hiccup every   */
  }                                     /* 49 days... horrible...            */

dwOffset += dwRest;                     /* add eventual rest                 */
if (dwOffset > nTimerMSecs)             /* if another timer tick's due       */
  {
  dwLastTime = dwCurTime;               /* remember this tick                */
  dwRest = dwOffset - nTimerMSecs;      /* calculate new rest                */
  pWorkThread->Process(emptyBuf,        /* process empty samples             */
                       (44100 * nTimerMSecs / 1000),
                       2,
                       dwCurTime - dwStartStamp);
  }
}

/*****************************************************************************/
/* LoadWaveInDevice : loads a Wave In device                                 */
/*****************************************************************************/

BOOL CVsthostApp::LoadWaveInDevice(CString sDevice, int nBufSz)
{
BOOL bRC = FALSE;
int nDevType = 0;

                                        /* either run with Wave In           */
CString sInDev = GetProfileString("Settings", "WaveIn");
nDevType = 0;
if (!sInDev.Left(5).CompareNoCase("MME: "))
  sInDev = sInDev.Mid(5);
else if (!sInDev.Left(6).CompareNoCase("ASIO: "))
  {
  nDevType = 1;
  sInDev = sInDev.Mid(6);
  }
else                                    /* default to correct old version    */
  {
  if (!sInDev.IsEmpty())
    WriteProfileString("Settings", "WaveIn", CString("MME: ") + sInDev);
  }
switch (nDevType)
  {
  case 0 :                              /* MME ?                             */
    WaveIn.SetupAllocSize(44100 / nBufSz, nBufSz);
    bRC = WaveIn.Open(sInDev,
                      NULL,
                      WAVEOPENDEFAULT,
                      WAVEOPENDEFAULT,
                      CALLBACK_EVENT);
    WaveIn.SetWorkThread(pWorkThread);
    break;
  case 1 :                              /* ASIO ?                            */
    // done in LoadWaveOutDevice()
    break;
  }

return bRC;
}

/*****************************************************************************/
/* LoadWaveOutDevice : loads a Wave Out device                               */
/*****************************************************************************/

BOOL CVsthostApp::LoadWaveOutDevice(CString sDevice, int &nBufSz)
{
BOOL bRC = FALSE;
int nDevType = 0;

if (!sDevice.Left(5).CompareNoCase("MME: "))
  sDevice = sDevice.Mid(5);
else if (!sDevice.Left(6).CompareNoCase("ASIO: "))
  {
  nDevType = 1;
  sDevice = sDevice.Mid(6);
  }
else                                    /* default to correct old version    */
  {
  if (!sDevice.IsEmpty())
    WriteProfileString("Settings", "WaveOut", CString("MME: ") + sDevice);
  }
switch (nDevType)
  {
  case 0 :                              /* MME ?                             */
    bRC = WaveOut.Open(sDevice,
                 NULL,
                 WAVEOPENDEFAULT,
                 WAVEOPENDEFAULT,
                 CALLBACK_EVENT);
    WaveOut.AllocateBuffers(44100 / nBufSz);
    break;
  case 1 :                              /* ASIO ?                            */
#ifdef __ASIO_H
    if (pAsioHost)
      bRC = pAsioHost->LoadDriver((char *)(LPCSTR)sDevice, nBufSz);
#endif
    break;
  }
return bRC;
}

/*****************************************************************************/
/* OnWavedev : called to configure the wave devices                          */
/*****************************************************************************/

void CVsthostApp::OnWavedev()
{
CWaveDeviceDialog dlg(AfxGetMainWnd());

dlg.InName = GetProfileString("Settings", "WaveIn");
dlg.OutName = GetProfileString("Settings", "WaveOut");
dlg.nBufSize = GetProfileInt("Settings", "WaveOutBufSize", 4410);
if (dlg.DoModal() == IDOK)
  {
  BOOL bWasRunning = bEngRunning;       /* remember old setting              */
  EngineStop();                         /* stop input engine                 */

  if (dlg.OutName.Left(6).CompareNoCase("ASIO: "))
    {
    TRACE0("(re-)Open Wave Input Device\n");
                                        /* open input device                 */
    if (!LoadWaveInDevice(dlg.InName, dlg.nBufSize))
      dlg.InName = "";                  /* upon error reset name             */
    }
  WriteProfileString("Settings", "WaveIn", dlg.InName);

  TRACE0("(re-)Open Wave Output Device\n");
                                        /* open output device                */
  if (!LoadWaveOutDevice(dlg.OutName, dlg.nBufSize))
    dlg.OutName = "";
  WriteProfileString("Settings", "WaveOut", dlg.OutName);
  WriteProfileInt("Settings", "WaveOutBufSize", dlg.nBufSize);

  if (bWasRunning)                      /* if it was running before,         */
    EngineStart();                      /* start input engine                */
  }
}

/*****************************************************************************/
/* SetupLatency : we need some latency to make the system work without hiccup*/
/*****************************************************************************/

void CVsthostApp::SetupLatency()
{
if (!WaveOut.IsOpen())                  /* only do if output open            */
  return;

#if 0
// works without that as well.
for (int i = 0; i < 2; i++)             /* prepare a bit                     */
  pWorkThread->Process(emptyBuf, 2205); /* of empty samples                  */
#endif
}

/*****************************************************************************/
/* EngineStop : stops eventual running input engine                          */
/*****************************************************************************/

void CVsthostApp::EngineStop()
{
if (!bEngRunning)
  return;

TRACE0("Stopping VSTHost engine\n");

SetTimer(0);                            /* close eventually opened HR timer  */
WaveIn.Stop();                          /* stop eventual record mode         */
MidiIn.Stop();
WaveIn.Reset();                         /* and make sure it doesn't pump on  */
MidiIn.Reset();

#ifdef __ASIO_H
if (pAsioHost)
  pAsioHost->Stop();
#endif

while ((WaveIn.WaitingBuffers()) ||     /* wait until all work done          */
       (WaveOut.WaitingBuffers()))
  {
  TRACE2("Waiting for buffer completion (%d in, %d out)\n",
         WaveIn.WaitingBuffers(),
         WaveOut.WaitingBuffers());
  Sleep (50);
  }
TRACE0("VSTHost Engine stopped.\n");
bEngRunning = FALSE;
}

/*****************************************************************************/
/* EngineStart : restarts input engine                                       */
/*****************************************************************************/

void CVsthostApp::EngineStart()
{
if (bEngRunning)
  return;

TRACE0("Starting VSTHost engine\n");

int nTimer = WaveOut.AllocatedBuffers();
if (nTimer)
  nTimer = 1000 / nTimer;
else
  nTimer = 44100 / GetProfileInt("Settings", "WaveOutBufSize", 4410);

#ifdef __ASIO_H
if (pAsioHost && pAsioHost->IsLoaded())
  {
  pAsioHost->Start();
  dwStartStamp = timeGetTime();         /* put common timestamp reference    */
  }
else
  {
#endif
  
  SetupLatency();                       /* setup latency                     */
  if (!WaveIn.Start())                  /* start recording (again)           */
    SetTimer(nTimer);                   /* upon error start HR timer         */
  dwStartStamp = timeGetTime();         /* put common timestamp reference    */
  WaveIn.SetStartStamp(dwStartStamp);

#ifdef __ASIO_H
  }
#endif

MidiIn.Start();
#if 0
// only works with message-based MIDI input!
MidiIn.SetStartStamp(dwStartStamp);     /* into all input devices            */
#endif

TRACE0("VSTHost engine up again\n");
bEngRunning = TRUE;
}

/*****************************************************************************/
/* FullStop : stops engine and closes devices                                */
/*****************************************************************************/

void CVsthostApp::FullStop()
{
TRACE0("Full stop start.\n");
EngineStop();                           /* stop input engine                 */

MidiIn.Close();                         /* close eventual opened MIDI devices*/
MidiOut.Close();
WaveIn.Close();                         /* close eventual opened Wave devices*/
WaveOut.Close();

#ifdef __ASIO_H
if (pAsioHost)
  pAsioHost->Exit();
#endif

TRACE0("Full stop ended.\n");
}

/*****************************************************************************/
/* OnEngStart : called to start the engine                                   */
/*****************************************************************************/

void CVsthostApp::OnEngStart() 
{
if (bEngRunning)
  EngineStop();
else
  EngineStart();
WriteProfileInt("Settings", "Engine", bEngRunning);
}

/*****************************************************************************/
/* OnUpdateEngStart : called to update the visual appearance                 */
/*****************************************************************************/

void CVsthostApp::OnUpdateEngStart(CCmdUI* pCmdUI) 
{
pCmdUI->SetCheck(bEngRunning);
}

/*****************************************************************************/
/* OnEngineRestart : called when the engine needs a restart                  */
/*****************************************************************************/

void CVsthostApp::OnEngineRestart() 
{
if (bEngRunning)
  {
  EngineStop();
  AfxGetMainWnd()->PostMessage(WM_COMMAND, IDM_ENG_START);
  }
}

/*****************************************************************************/
/* OnUpdateEngineRestart : called to update the visual appearance            */
/*****************************************************************************/

void CVsthostApp::OnUpdateEngineRestart(CCmdUI* pCmdUI) 
{
pCmdUI->Enable(bEngRunning);
}

/*****************************************************************************/
/* OnAsioCpl : displays the ASIO Control Panel, if ASIO driver loaded        */
/*****************************************************************************/

void CVsthostApp::OnAsioCpl() 
{
#ifdef __ASIO_H
if ((pAsioHost) && (pAsioHost->IsLoaded()))
  pAsioHost->ControlPanel();
#endif	
}

/*****************************************************************************/
/* OnUpdateAsioCpl : updates the visual appearance                           */
/*****************************************************************************/

void CVsthostApp::OnUpdateAsioCpl(CCmdUI* pCmdUI) 
{
#ifdef __ASIO_H
pCmdUI->Enable((pAsioHost) && (pAsioHost->IsLoaded()));
#else
pCmdUI->Enable(FALSE);
#endif
}

/*****************************************************************************/
/* SendResult : sends out calculated buffers to current output device        */
/*****************************************************************************/

void CVsthostApp::SendResult(float **pBuffers, int nLength)
{
#ifdef __ASIO_H
if ((pAsioHost) && (pAsioHost->IsLoaded()))
  pAsioHost->OnProcessed(pBuffers);
else
#endif
  WaveOut.SendData(pBuffers, nLength);
}
